@@ -24,7 +24,8 @@ __all__ = [ "ConfigError", "ConstantsModule", "Executable", "Freezer" ]
 EXTENSION_LOADER_SOURCE = \
 """
 def __bootstrap__():
-    import imp, os, sys
+    import imp, sys
+    os = sys.modules['os']
     global __bootstrap__, __loader__
     __loader__ = None; del __bootstrap__, __loader__
 
@@ -183,9 +184,10 @@ class Freezer(object):
         if includeMode:
             shutil.copymode(source, target)
         self.filesCopied[normalizedTarget] = None
-        if copyDependentFiles:
+        if copyDependentFiles \
+                and source not in self.finder.excludeDependentFiles:
             for source in self._GetDependentFiles(source):
-                target = os.path.join(self.targetDir, os.path.basename(source))
+                target = os.path.join(targetDir, os.path.basename(source))
                 self._CopyFile(source, target, copyDependentFiles)
 
     def _CreateDirectory(self, path):
@@ -448,10 +450,10 @@ class Freezer(object):
         if module.path is None or module.file is None:
             return False
         if self.zipIncludeAllPackages \
-                and module.name not in self.zipExcludePackages:
-            return True
-        return self.zipExcludeAllPackages \
-                and module.name not in self.zipIncludePackages
+                and module.name not in self.zipExcludePackages \
+                or module.name in self.zipIncludePackages:
+            return False
+        return True
 
     def _VerifyConfiguration(self):
         if self.compress is None:
@@ -494,13 +496,6 @@ class Freezer(object):
         finder.ReportMissingModules()
 
         targetDir = os.path.dirname(fileName)
-        if sys.platform != "win32":
-            if os.path.basename(targetDir) == "lib64":
-                linkName = os.path.join(os.path.dirname(targetDir), "lib")
-                if not os.path.exists(linkName):
-                    os.symlink("lib64", linkName)
-            targetDir = os.path.join(targetDir,
-                    "python%s.%s" % sys.version_info[:2])
         self._CreateDirectory(targetDir)
 
         # Prepare zip file
@@ -555,13 +550,18 @@ class Freezer(object):
             # the file is up to date so we can safely set this value to zero
             if module.code is not None:
                 if module.file is not None and os.path.exists(module.file):
-                    mtime = os.stat(module.file).st_mtime
+                   stat = os.stat(module.file)
+                   mtime = stat.st_mtime
+                   size = stat.st_size & 0xFFFFFFFF
                 else:
                     mtime = time.time()
+                    size = 0
                 if sys.version_info[:2] < (3, 3):
                     header = magic + struct.pack("<i", int(mtime))
+                elif sys.version_info[:2] == (3, 7):
+                    header = magic + struct.pack("<iii",0, int(mtime), size)
                 else:
-                    header = magic + struct.pack("<ii", int(mtime), 0)
+                    header = magic + struct.pack("<ii", int(mtime), size)
                 data = header + marshal.dumps(module.code)
 
             # if the module should be written to the file system, do so
@@ -621,12 +621,9 @@ class Freezer(object):
         self.finder = self._GetModuleFinder()
         for executable in self.executables:
             self._FreezeExecutable(executable)
-        targetDir = zipTargetDir = self.targetDir
-        rawLibDir = distutils.sysconfig.get_config_var("LIBDIR")
-        if rawLibDir:
-            zipTargetDir = os.path.join(targetDir, os.path.basename(rawLibDir))
-        fileName = os.path.join(zipTargetDir,
-                "python%s%s.zip" % sys.version_info[:2])
+        targetDir = self.targetDir
+        zipTargetDir = os.path.join(self.targetDir, "lib")
+        fileName = os.path.join(zipTargetDir, "library.zip")
         self._RemoveFile(fileName)
         self._WriteModules(fileName, self.finder)
 
